/**
 * @vitest-environment-options { "xmlPath": "./tests/resources/GP_Temperature_Simplified.xml" }
 */

import { select, toTextArr, toTextContentArr } from "../../util";
import { StatementMetaDataType, DCCDocument } from "../../../src";

const base = "/dcc:digitalCalibrationCertificate/dcc:administrativeData/dcc:statements";
const norm_base = `${base}/dcc:statement[1]`;
const basic_validityRange_base = `${base}/dcc:statement[2]`;
const basic_conformity_base = `${base}/dcc:statement[3]`;
const basic_recalibration_base = `${base}/dcc:statement[4]`;

const xpath = {
  statements: {
    norm: {
      norm: `string(${norm_base}/dcc:norm)`,
      reference: `string(${norm_base}/dcc:reference)`,
      declaration: {
        content: `${norm_base}/dcc:declaration/dcc:content`,
      },
      refType: `string(${norm_base}/@refType)`,
    },
    basic_validityRange: {
      declaration: {
        content: `${basic_validityRange_base}/dcc:declaration/dcc:content`,
      },
      data: {
        quantity1: {
          name: {
            content: `${basic_validityRange_base}/dcc:data/dcc:quantity[1]/dcc:name/dcc:content`,
          },
          si_real: {
            si_value: `string(${basic_validityRange_base}/dcc:data/dcc:quantity[1]/si:real/si:value)`,
            si_unit: `string(${basic_validityRange_base}/dcc:data/dcc:quantity[1]/si:real/si:unit)`,
          },
        },
        quantity2: {
          name: {
            content: `${basic_validityRange_base}/dcc:data/dcc:quantity[2]/dcc:name/dcc:content`,
          },
          si_real: {
            si_value: `string(${basic_validityRange_base}/dcc:data/dcc:quantity[2]/si:real/si:value)`,
            si_unit: `string(${basic_validityRange_base}/dcc:data/dcc:quantity[2]/si:real/si:unit)`,
          },
        },
      },
      refType: `string(${basic_validityRange_base}/@refType)`,
    },
    basic_conformity: {
      declaration: {
        content: `${basic_conformity_base}/dcc:declaration/dcc:content`,
      },
      respAuthority: {
        name: {
          content: `${basic_conformity_base}/dcc:respAuthority/dcc:name/dcc:content`,
        },
        email: `string(${basic_conformity_base}/dcc:respAuthority/dcc:eMail)`,
        location: {
          city: `string(${basic_conformity_base}/dcc:respAuthority/dcc:location/dcc:city)`,
          countryCode: `string(${basic_conformity_base}/dcc:respAuthority/dcc:location/dcc:countryCode)`,
          postCode: `string(${basic_conformity_base}/dcc:respAuthority/dcc:location/dcc:postCode)`,
        },
      },
      conformity: `string(${basic_conformity_base}/dcc:conformity)`,
      refType: `string(${basic_conformity_base}/@refType)`,
    },
    basic_recalibration: {
      declaration: {
        content: `${basic_recalibration_base}/dcc:declaration/dcc:content`,
      },
      date: `string(${basic_recalibration_base}/dcc:date)`,
      respAuthority: {
        name: {
          content: `${basic_recalibration_base}/dcc:respAuthority/dcc:name/dcc:content`,
        },
        email: `string(${basic_recalibration_base}/dcc:respAuthority/dcc:eMail)`,
        location: {
          city: `string(${basic_recalibration_base}/dcc:respAuthority/dcc:location/dcc:city)`,
          countryCode: `string(${basic_recalibration_base}/dcc:respAuthority/dcc:location/dcc:countryCode)`,
          postCode: `string(${basic_recalibration_base}/dcc:respAuthority/dcc:location/dcc:postCode)`,
        },
      },
      refType: `string(${basic_recalibration_base}/@refType)`,
    },
  },
};

describe("GP_DCC_Temperature_Simplified: StatementMetaDataType", () => {
  let dcc: DCCDocument,
    norm: StatementMetaDataType,
    basic_validityRange: StatementMetaDataType,
    basic_conformity: StatementMetaDataType,
    basic_recalibration: StatementMetaDataType,
    dom;

  beforeEach(() => {
    ({ dcc, dom } = recreateEnv());
    norm = dcc.digitalCalibrationCertificate.administrativeData.statements.statement[0];
    basic_validityRange = dcc.digitalCalibrationCertificate.administrativeData.statements.statement[1];
    basic_conformity = dcc.digitalCalibrationCertificate.administrativeData.statements.statement[2];
    basic_recalibration = dcc.digitalCalibrationCertificate.administrativeData.statements.statement[3];
  });

  // region norm
  test("should get correct norm declaration content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.norm.declaration.content, dom);
    expect(toTextArr(norm.declaration.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct norm model from XML", () => {
    expect(norm.norm[0]._text).toBe(select(xpath.statements.norm.norm, dom));
  });

  test("should get correct norm reference from XML", () => {
    expect(norm.reference[0]._text).toBe(select(xpath.statements.norm.reference, dom));
  });
  // endregion

  // region basic_validityRange
  test("should get correct basic_validityRange declaration content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.basic_validityRange.declaration.content, dom);
    expect(toTextArr(basic_validityRange.declaration.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_validityRange data quantity 1 name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.basic_validityRange.data.quantity1.name.content, dom);
    expect(toTextArr(basic_validityRange.data.quantity[0].name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_validityRange data quantity 1 si:value from XML", () => {
    expect(basic_validityRange.data.quantity[0].real.value._text).toBe(select(xpath.statements.basic_validityRange.data.quantity1.si_real.si_value, dom));
  });

  test("should get correct basic_validityRange data quantity 2 si:unit from XML", () => {
    expect(basic_validityRange.data.quantity[0].real.unit._text).toBe(select(xpath.statements.basic_validityRange.data.quantity1.si_real.si_unit, dom));
  });

  test("should get correct basic_validityRange data quantity 2 name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.basic_validityRange.data.quantity2.name.content, dom);
    expect(toTextArr(basic_validityRange.data.quantity[1].name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_validityRange data quantity 2 si:value from XML", () => {
    expect(basic_validityRange.data.quantity[1].real.value._text).toBe(select(xpath.statements.basic_validityRange.data.quantity2.si_real.si_value, dom));
  });

  test("should get correct basic_validityRange data quantity 2 si:unit from XML", () => {
    expect(basic_validityRange.data.quantity[1].real.unit._text).toBe(select(xpath.statements.basic_validityRange.data.quantity2.si_real.si_unit, dom));
  });

  test("should get correct basic_validityRange refType from XML", () => {
    expect(basic_validityRange._attr.refType).toBe(select(xpath.statements.basic_validityRange.refType, dom));
  });
  // endregion

  // region basic_conformity
  test("should get correct basic_conformity declaration content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.basic_conformity.declaration.content, dom);
    expect(toTextArr(basic_conformity.declaration.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_conformity responsible authority name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.basic_conformity.respAuthority.name.content, dom);
    expect(toTextArr(basic_conformity.respAuthority.name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_conformity responsible authority email from XML", () => {
    expect(basic_conformity.respAuthority.eMail._text).toBe(select(xpath.statements.basic_conformity.respAuthority.email, dom));
  });

  test("should get correct basic_conformity responsible authority location city from XML", () => {
    expect(basic_conformity.respAuthority.location.city[0]._text).toBe(select(xpath.statements.basic_conformity.respAuthority.location.city, dom));
  });

  test("should get correct basic_conformity responsible authority location country code from XML", () => {
    expect(basic_conformity.respAuthority.location.countryCode[0]._text).toBe(
      select(xpath.statements.basic_conformity.respAuthority.location.countryCode, dom),
    );
  });

  test("should get correct basic_conformity responsible authority location post code from XML", () => {
    expect(basic_conformity.respAuthority.location.postCode[0]._text).toBe(select(xpath.statements.basic_conformity.respAuthority.location.postCode, dom));
  });

  test("should get correct basic_conformity conformity from XML", () => {
    expect(basic_conformity.conformity._text).toBe(select(xpath.statements.basic_conformity.conformity, dom));
  });
  // endregion

  // region basic_recalibration
  test("should get correct basic_recalibration declaration content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.basic_recalibration.declaration.content, dom);
    expect(toTextArr(basic_recalibration.declaration.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_recalibration refType from XML", () => {
    expect(basic_recalibration.date._text).toBe(select(xpath.statements.basic_recalibration.date, dom));
  });

  /* responsible authority 2 */
  test("should get correct basic_recalibration responsible authority name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.basic_recalibration.respAuthority.name.content, dom);
    expect(toTextArr(basic_recalibration.respAuthority.name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_recalibration responsible authority email from XML", () => {
    expect(basic_recalibration.respAuthority.eMail._text).toBe(select(xpath.statements.basic_recalibration.respAuthority.email, dom));
  });

  test("should get correct basic_recalibration responsible authority location city from XML", () => {
    expect(basic_recalibration.respAuthority.location.city[0]._text).toBe(select(xpath.statements.basic_recalibration.respAuthority.location.city, dom));
  });
  test("should get correct basic_recalibration responsible authority location country code from XML", () => {
    expect(basic_recalibration.respAuthority.location.countryCode[0]._text).toBe(
      select(xpath.statements.basic_recalibration.respAuthority.location.countryCode, dom),
    );
  });
  test("should get correct basic_recalibration responsible authority location post code from XML", () => {
    expect(basic_recalibration.respAuthority.location.postCode[0]._text).toBe(
      select(xpath.statements.basic_recalibration.respAuthority.location.postCode, dom),
    );
  });

  test("should get correct basic_recalibration refType from XML", () => {
    expect(basic_recalibration._attr.refType).toBe(select(xpath.statements.basic_recalibration.refType, dom));
  });
  // endregion

  /* TODO: setters */
});
