/**
 * @vitest-environment-options { "xmlPath": "./tests/resources/GP_Temperature_Simplified.xml" }
 */

import { select, toTextArr, toTextContentArr } from "../../util";
import { ConditionType, DCCDocument } from "../../../src";

const base = "/dcc:digitalCalibrationCertificate/dcc:measurementResults/dcc:measurementResult[1]/dcc:influenceConditions";
const gp_immersionDepth_base = `${base}/dcc:influenceCondition[1]`;
const basic_temperature_base = `${base}/dcc:influenceCondition[2]`;
const basic_humidityRelative_base = `${base}/dcc:influenceCondition[3]`;

const xpath = {
  influenceConditions: {
    gp_immersionDepth: {
      name: {
        content: `${gp_immersionDepth_base}/dcc:name/dcc:content`,
      },
      data: {
        quantity: {
          name: {
            content: `${gp_immersionDepth_base}/dcc:data/dcc:quantity[1]/dcc:name/dcc:content`,
          },
          si_real: {
            si_value: `string(${gp_immersionDepth_base}/dcc:data/dcc:quantity[1]/si:real/si:value)`,
            si_unit: `string(${gp_immersionDepth_base}/dcc:data/dcc:quantity[1]/si:real/si:unit)`,
          },
        },
      },
      refType: `string(${gp_immersionDepth_base}//@refType)`,
    },
    basic_temperature: {
      name: {
        content: `${basic_temperature_base}/dcc:name/dcc:content`,
      },
      description: {
        content: `${basic_temperature_base}/dcc:description/dcc:content`,
      },
      data: {
        quantity1: {
          name: {
            content: `${basic_temperature_base}/dcc:data/dcc:quantity[1]/dcc:name/dcc:content`,
          },
          si_real: {
            si_value: `string(${basic_temperature_base}/dcc:data/dcc:quantity[1]/si:real/si:value)`,
            si_unit: `string(${basic_temperature_base}/dcc:data/dcc:quantity[1]/si:real/si:unit)`,
          },
          refType: `string(${basic_temperature_base}/dcc:data/dcc:quantity[1]//@refType)`,
        },
        quantity2: {
          name: {
            content: `${basic_temperature_base}/dcc:data/dcc:quantity[2]/dcc:name/dcc:content`,
          },
          si_real: {
            si_value: `string(${basic_temperature_base}/dcc:data/dcc:quantity[2]/si:real/si:value)`,
            si_unit: `string(${basic_temperature_base}/dcc:data/dcc:quantity[2]/si:real/si:unit)`,
          },
          refType: `string(${basic_temperature_base}/dcc:data/dcc:quantity[2]//@refType)`,
        },
      },
      refType: `string(${basic_temperature_base}//@refType)`,
    },
    basic_humidityRelative: {
      name: {
        content: `${basic_humidityRelative_base}/dcc:name/dcc:content`,
      },
      description: {
        content: `${basic_humidityRelative_base}/dcc:description/dcc:content`,
        contentDE: `${basic_humidityRelative_base}/dcc:description/dcc:content[@lang='de']`,
        contentEN: `${basic_humidityRelative_base}/dcc:description/dcc:content[@lang='en']`,
      },
      data: {
        quantity1: {
          name: {
            content: `${basic_humidityRelative_base}/dcc:data/dcc:quantity[1]/dcc:name/dcc:content`,
          },
          si_real: {
            si_value: `string(${basic_humidityRelative_base}/dcc:data/dcc:quantity[1]/si:real/si:value)`,
            si_unit: `string(${basic_humidityRelative_base}/dcc:data/dcc:quantity[1]/si:real/si:unit)`,
          },
          refType: `string(${basic_humidityRelative_base}/dcc:data/dcc:quantity[1]//@refType)`,
        },
        quantity2: {
          name: {
            content: `${basic_humidityRelative_base}/dcc:data/dcc:quantity[2]/dcc:name/dcc:content`,
          },
          si_real: {
            si_value: `string(${basic_humidityRelative_base}/dcc:data/dcc:quantity[2]/si:real/si:value)`,
            si_unit: `string(${basic_humidityRelative_base}/dcc:data/dcc:quantity[2]/si:real/si:unit)`,
          },
          refType: `string(${basic_humidityRelative_base}/dcc:data/dcc:quantity[2]//@refType)`,
        },
      },
      refType: `string(${basic_humidityRelative_base}//@refType)`,
    },
  },
};

describe("GP_DCC_Temperature_Simplified: InfluenceConditionListType", () => {
  let dcc: DCCDocument, gp_immersionDepth: ConditionType, basic_temperature: ConditionType, basic_humidityRelative: ConditionType, dom;

  beforeEach(() => {
    ({ dcc, dom } = recreateEnv());
    const influenceConditions = dcc.digitalCalibrationCertificate.measurementResults.measurementResult[0].influenceConditions;
    gp_immersionDepth = influenceConditions.influenceCondition[0];
    basic_temperature = influenceConditions.influenceCondition[1];
    basic_humidityRelative = influenceConditions.influenceCondition[2];
  });

  // region gp_immersionDepth
  test("should get correct gp_immersionDepth name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.influenceConditions.gp_immersionDepth.name.content, dom);
    expect(toTextArr(gp_immersionDepth.name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct gp_immersionDepth quantity name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.influenceConditions.gp_immersionDepth.data.quantity.name.content, dom);
    expect(toTextArr(gp_immersionDepth.data.quantity[0].name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct gp_immersionDepth quantity si:value from XML", () => {
    expect(gp_immersionDepth.data.quantity[0].real.value._text).toBe(select(xpath.influenceConditions.gp_immersionDepth.data.quantity.si_real.si_value, dom));
  });

  test("should get correct gp_immersionDepth quantity si:unit from XML", () => {
    expect(gp_immersionDepth.data.quantity[0].real.unit._text).toBe(select(xpath.influenceConditions.gp_immersionDepth.data.quantity.si_real.si_unit, dom));
  });

  test("should get correct gp_immersionDepth refType from XML", () => {
    expect(gp_immersionDepth._attr.refType).toBe(select(xpath.influenceConditions.gp_immersionDepth.refType, dom));
  });
  // endregion

  // region basic_temperature
  test("should get correct basic_temperature name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.influenceConditions.basic_temperature.name.content, dom);
    expect(toTextArr(basic_temperature.name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_temperature description content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.influenceConditions.basic_temperature.description.content, dom);
    expect(toTextArr(basic_temperature.description.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_temperature quantity name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.influenceConditions.basic_temperature.data.quantity1.name.content, dom);
    expect(toTextArr(basic_temperature.data.quantity[0].name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_temperature quantity si:value from XML", () => {
    expect(basic_temperature.data.quantity[0].real.value._text).toBe(select(xpath.influenceConditions.basic_temperature.data.quantity1.si_real.si_value, dom));
  });

  test("should get correct basic_temperature quantity si:unit from XML", () => {
    expect(basic_temperature.data.quantity[0].real.unit._text).toBe(select(xpath.influenceConditions.basic_temperature.data.quantity1.si_real.si_unit, dom));
  });

  test("should get correct basic_temperature quantity 1 refType from XML", () => {
    expect(basic_temperature.data.quantity[0]._attr.refType).toBe(select(xpath.influenceConditions.basic_temperature.data.quantity1.refType, dom));
  });

  test("should get correct basic_temperature quantity 2 si:value from XML", () => {
    expect(basic_temperature.data.quantity[1].real.value._text).toBe(select(xpath.influenceConditions.basic_temperature.data.quantity2.si_real.si_value, dom));
  });

  test("should get correct basic_temperature quantity 2 si:unit from XML", () => {
    expect(basic_temperature.data.quantity[1].real.unit._text).toBe(select(xpath.influenceConditions.basic_temperature.data.quantity2.si_real.si_unit, dom));
  });

  test("should get correct basic_temperature quantity 2 refType from XML", () => {
    expect(basic_temperature.data.quantity[1]._attr.refType).toBe(select(xpath.influenceConditions.basic_temperature.data.quantity2.refType, dom));
  });

  test("should get correct basic_temperature refType from XML", () => {
    expect(basic_temperature._attr.refType).toBe(select(xpath.influenceConditions.basic_temperature.refType, dom));
  });
  // endregion

  // region basic_humidityRelative
  test("should get correct basic_humidityRelative name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.influenceConditions.basic_humidityRelative.name.content, dom);
    expect(toTextArr(basic_humidityRelative.name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_humidityRelative description content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.influenceConditions.basic_humidityRelative.description.content, dom);
    expect(toTextArr(basic_humidityRelative.description.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_humidityRelative quantity 1 name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.influenceConditions.basic_humidityRelative.data.quantity1.name.content, dom);
    expect(toTextArr(basic_humidityRelative.data.quantity[0].name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_humidityRelative quantity 1 si:value from XML", () => {
    expect(basic_humidityRelative.data.quantity[0].real.value._text).toBe(
      select(xpath.influenceConditions.basic_humidityRelative.data.quantity1.si_real.si_value, dom),
    );
  });

  test("should get correct basic_humidityRelative quantity 1 si:unit from XML", () => {
    expect(basic_humidityRelative.data.quantity[0].real.unit._text).toBe(
      select(xpath.influenceConditions.basic_humidityRelative.data.quantity1.si_real.si_unit, dom),
    );
  });

  test("should get correct basic_humidityRelative quantity 1 refType from XML", () => {
    expect(basic_humidityRelative.data.quantity[0]._attr.refType).toBe(select(xpath.influenceConditions.basic_humidityRelative.data.quantity1.refType, dom));
  });

  test("should get correct basic_humidityRelative quantity 2 name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.influenceConditions.basic_humidityRelative.data.quantity2.name.content, dom);
    expect(toTextArr(basic_humidityRelative.data.quantity[1].name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct basic_humidityRelative quantity 2 si:value from XML", () => {
    expect(basic_humidityRelative.data.quantity[1].real.value._text).toBe(
      select(xpath.influenceConditions.basic_humidityRelative.data.quantity2.si_real.si_value, dom),
    );
  });

  test("should get correct basic_humidityRelative quantity 2 si:unit from XML", () => {
    expect(basic_humidityRelative.data.quantity[1].real.unit._text).toBe(
      select(xpath.influenceConditions.basic_humidityRelative.data.quantity2.si_real.si_unit, dom),
    );
  });

  test("should get correct basic_humidityRelative quantity 2 refType from XML", () => {
    expect(basic_humidityRelative.data.quantity[1]._attr.refType).toBe(select(xpath.influenceConditions.basic_humidityRelative.data.quantity2.refType, dom));
  });

  test("should get correct basic_humidityRelative refType from XML", () => {
    expect(basic_humidityRelative._attr.refType).toBe(select(xpath.influenceConditions.basic_humidityRelative.refType, dom));
  });
  // endregion

  /* TODO: setters */
});
